function dat = IterMLEsolve(specdat,tdat,varargin)
% function dat = IterMLEsolve(specdat,tdat,varargin)
%    Solve for A-space line integrals using maximum likelihood.
% This version assumes counting detectors. Uses Poisson distribution. Assumes independent data for each spectrum
%  Assumes PHA with nbins specified by idx_threshold array in specdat struct
%  Approach:
% ToDo: --> enter this. See Onenote keyword mle_poisson
%    inputs:
%      specdat: a spectrum data struct for incident spectrum with fields
%        egy: x-ray energies
%        specnum: the photon number spectrum
%        mus: the basis materials attenuation coefficients at specified energies. 
%        idx_threshold: indices into the spectrum array for inter-bin energies
%      tdat: a struct with test data with fields
%        ns: a matrix of count values. Values for spectra along rows, 1 row per test set of thicknesses
%        As: the A values for each row of tdat.ns, will use for computing errors: As(1) + 1i*As(2)
%      verbose: (optional = false) If present, prints out debug info
%    outputs:
%      dat: a struct with fields
%        cdat,tdat--the input data
%        Astest: the A values for the test data (tdat)
%        errs: the std dev of the errors with tdat
%        solvedat: a struct with parameters so can fit other sets of data without re-doing the calibration
%           See code for other fields
%    REA 4/20/10 - 8/Sep/2013 16:53
 
  % process the optional arguments
nreqargs = 2;
assert(nargin>=nreqargs);
solvedat = [];
no_random_offset = false;
get_debug_data = false;
verbose = false;
if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'no_random_offset';          no_random_offset = true;
     case 'get_debug_data';          get_debug_data = true;
     case 'verbose';          verbose = true;
     case 'solvedat';         solvedat=varargin{i+1};  assert(isstruct(solvedat));   i=i+1;
     case 'nside';         nside=varargin{i+1};   i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

[negys,nbasis] = size(specdat.mus);

ns = tdat.ns;
[npoints_in,nbins] = size(ns);

  % use the solvedat struct to pass data into functions in fminsearch
solvedat.specdat = specdat;

s = warning('off', 'MATLAB:nearlySingularMatrix');

options= optimset('Display', 'off','TolFun',0.0001); % used in fminsearch
Astest = zeros(npoints_in,nbasis); % will hold solution points
As_init_used = Astest; % initial guess, set equal to Astest to pre-declare same size
for kpnt = 1:npoints_in
    ns = tdat.ns(kpnt,:);
    solvedat.ns = ns;
    Ainit = tdat.As(kpnt,:) + 0.1*(rand(1,nbasis) - 0.5 ); % init point is random offset from actual
    As_init_used(kpnt,:) = Ainit;
    [Aopt, neg_prob_min,exitflag,output] = fminsearch(@(A) NegLikelihoodFuncLoc(A,ns,specdat), Ainit ,options);
    Astest(kpnt,:) = Aopt;
    if verbose, fprintf('.'); end
end % for kpnt = 1:npnts
if verbose, fprintf('\n'); end

dat.As_init = As_init_used;
dat.Astest = Astest;
dat.Asdiff = tdat.As - dat.Astest;
solvedat.specdat = specdat;
solvedat.tdat = tdat;
dat.solvedat = solvedat;

    %------------ Local Functions ---------------
        function [p,parms] = NegLikelihoodFuncLoc(As,n,specdat)
        % function [p,parms] = NegLikelihoodFuncLoc(As,solvedat)
        %  Compute negative of probability (so can maximize)
        %  inputs:
        %     As: a 2 column matrix with As arranged along rows
        %     n: the measured data
        %      specdat: spectrum struct for incident spectrum
        %  outputs:
        %      p: the negative of the logarithm of probability
        %      parms: a struct with data used in calculation
        % Approach: compute probability from lambdas using poisspdf
        
        lambdas = PHAbinCounts(As,specdat,specdat.idx_threshold);
        probabilities = poisspdf(repmat(round(n),size(As,1),1),lambdas); % probabilities of each n value
        
        p = -log(prod(probabilities,2)); % assume nvalues are independent
        % for debug disp([As lambdas probabilities])
        parms.p = p;
        parms.lambdas = lambdas;

 
