function dat = PolyASolveEquations(cdat,tdat,varargin)
% function dat = PolyASolveEquations(cdat,tdat,varargin)
%    Solve for A-space line integrals using fit to linear and 2nd order polynomial
%    inputs:
%      cdat: a struct with calibration data with fields
%        logI: a matrix of log(I/I0) with one column for each spectrum
%        As: the A values for each row of cdat.logI
%      tdat: a struct with test data with fields
%        logI: a matrix of log(I/I0) same number of columns as cdat
%        As: the A values for each row of cdat.logI
%      solvedat: (optional = []) If specified uses this struct to solve the eqns instead of doing nlinfits 
%      linear: (optional = false) If present uses linear fit only
%    outputs:
%      dat: a struct with fields
%        cdat,tdat--the input data
%        Astest: the A values for the test data (tdat)
%        solvedat: a struct with parameters so can fit other sets of data without re-doing the calibration
%           See code for fields
% Example of use
%    % create the input data structs
% cdat.logI = logI;
% cdat.As = ts;
% tdat = cdat; % will test on calibration data
%    % solve for fit params and test with tdat
% dat = PolyASolveEquations(cdat,tdat);
%   % solve again without re-calibrating
% dat2 = PolyASolveEquations([],tdat,'solvedat',dat.solvedat);
%    REA 6/26/09 
% 6/29/09  handle case with more than 2 spectra 
% 7/11/09 do linear case
nreqargs = 2;
assert(nargin>=nreqargs);
solvedat = [];
do_linear = false;

if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'linear';          do_linear = true;
     case 'solvedat';         solvedat=varargin{i+1};  assert(isstruct(solvedat));   i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

if isempty(solvedat)
       % fit to both components simultaneously
    if do_linear
      pcols = [];
    else
      pcols = ProdColumns(cdat.logI,2);
    end
    d = [ ones(size(cdat.logI,1),1) cdat.logI pcols];
    cspoly = d\ cdat.As;
    solvedat.Asfit = d*cspoly;
    solvedat.errs = std(cdat.As - solvedat.Asfit)./range(cdat.As);
    solvedat.cspoly = cspoly;
    solvedat.linear = do_linear;
end
dat.solvedat = solvedat;

dat.tdat = tdat;

if solvedat.linear
  pcols = [];
else
  pcols = ProdColumns(tdat.logI,2);
end

d = [ ones(size(tdat.logI,1),1) tdat.logI pcols];
   
dat.Astest = d*solvedat.cspoly;
dat.Asdiff = tdat.As - dat.Astest;
dat.errs = std(dat.Asdiff)./range(tdat.As);
dat.errs_raw =  std(tdat.As - dat.Astest);


        